@@ -150,7 +150,7 @@ struct connection *quic_sock_accept_conn(struct listener *l, int *status)
 	struct li_per_thread *lthr = &l->per_thr[tid];
 
 	qc = MT_LIST_POP(&lthr->quic_accept.conns, struct quic_conn *, accept_list);
-	if (!qc)
+	if (!qc || qc->flags & (QUIC_FL_CONN_CLOSING|QUIC_FL_CONN_DRAINING))
 		goto done;
 
 	if (!new_quic_cli_conn(qc, l, &qc->peer_addr))
@@ -199,33 +199,41 @@ struct task *quic_lstnr_dghdlr(struct task *t, void *ctx, unsigned int state)
 	return t;
 }
 
-/* Retrieve the DCID from the datagram found in <buf> and deliver it to the
+/* Retrieve the DCID from the datagram found at <pos> position and deliver it to the
  * correct datagram handler.
  * Return 1 if a correct datagram could be found, 0 if not.
  */
-static int quic_lstnr_dgram_dispatch(unsigned char *buf, size_t len, void *owner,
+static int quic_lstnr_dgram_dispatch(unsigned char *pos, size_t len, void *owner,
                                      struct sockaddr_storage *saddr,
                                      struct sockaddr_storage *daddr,
                                      struct quic_dgram *new_dgram, struct list *dgrams)
 {
 	struct quic_dgram *dgram;
-	const struct listener *l = owner;
 	unsigned char *dcid;
 	size_t dcid_len;
 	int cid_tid;
 
-	if (!len || !quic_get_dgram_dcid(buf, buf + len, &dcid, &dcid_len))
+	if (!len || !quic_get_dgram_dcid(pos, pos + len, &dcid, &dcid_len))
 		goto err;
 
 	dgram = new_dgram ? new_dgram : pool_alloc(pool_head_quic_dgram);
 	if (!dgram)
 		goto err;
 
-	cid_tid = quic_get_cid_tid(dcid, &l->rx);
+	if ((cid_tid = quic_get_cid_tid(dcid, dcid_len, saddr, pos, len)) < 0) {
+		/* Use the current thread if CID not found. If a clients opens
+		 * a connection with multiple packets, it is possible that
+		 * several threads will deal with datagrams sharing the same
+		 * CID. For this reason, the CID tree insertion will be
+		 * conducted as an atomic operation and the datagram ultimately
+		 * redispatch by the late thread.
+		 */
+		cid_tid = tid;
+	}
 
 	/* All the members must be initialized! */
 	dgram->owner = owner;
-	dgram->buf = buf;
+	dgram->buf = pos;
 	dgram->len = len;
 	dgram->dcid = dcid;
 	dgram->dcid_len = dcid_len;
@@ -254,12 +262,12 @@ static int quic_lstnr_dgram_dispatch(unsigned char *buf, size_t len, void *owner
  *
  * Returns the last unused datagram or NULL if no occurrence found.
  */
-static struct quic_dgram *quic_rxbuf_purge_dgrams(struct quic_receiver_buf *buf)
+static struct quic_dgram *quic_rxbuf_purge_dgrams(struct quic_receiver_buf *rbuf)
 {
 	struct quic_dgram *cur, *prev = NULL;
 
-	while (!LIST_ISEMPTY(&buf->dgram_list)) {
-		cur = LIST_ELEM(buf->dgram_list.n, struct quic_dgram *, recv_list);
+	while (!LIST_ISEMPTY(&rbuf->dgram_list)) {
+		cur = LIST_ELEM(rbuf->dgram_list.n, struct quic_dgram *, recv_list);
 
 		/* Loop until a not yet consumed datagram is found. */
 		if (HA_ATOMIC_LOAD(&cur->buf))
@@ -267,7 +275,7 @@ static struct quic_dgram *quic_rxbuf_purge_dgrams(struct quic_receiver_buf *buf)
 
 		/* Clear buffer of current unused datagram. */
 		LIST_DELETE(&cur->recv_list);
-		b_del(&buf->buf, cur->len);
+		b_del(&rbuf->buf, cur->len);
 
 		/* Free last found unused datagram. */
 		if (prev)
@@ -870,6 +878,15 @@ void qc_release_fd(struct quic_conn *qc, int reinit)
 	}
 }
 
+/* Wrapper for fd_want_recv(). Safe even if connection does not used its owned
+ * socket.
+ */
+void qc_want_recv(struct quic_conn *qc)
+{
+	if (qc_test_fd(qc))
+		fd_want_recv(qc->fd);
+}
+
 /*********************** QUIC accept queue management ***********************/
 /* per-thread accept queues */
 struct quic_accept_queue *quic_accept_queues;
@@ -879,8 +896,8 @@ struct quic_accept_queue *quic_accept_queues;
  */
 void quic_accept_push_qc(struct quic_conn *qc)
 {
-	struct quic_accept_queue *queue = &quic_accept_queues[qc->tid];
-	struct li_per_thread *lthr = &qc->li->per_thr[qc->tid];
+	struct quic_accept_queue *queue = &quic_accept_queues[tid];
+	struct li_per_thread *lthr = &qc->li->per_thr[tid];
 
 	/* early return if accept is already in progress/done for this
 	 * connection
@@ -904,7 +921,7 @@ void quic_accept_push_qc(struct quic_conn *qc)
 	MT_LIST_APPEND(&lthr->quic_accept.conns, &qc->accept_list);
 
 	/* 3. wake up the queue tasklet */
-	tasklet_wakeup(quic_accept_queues[qc->tid].tasklet);
+	tasklet_wakeup(quic_accept_queues[tid].tasklet);
 }
 
 /* Tasklet handler to accept QUIC connections. Call listener_accept on every
