@@ -189,15 +189,15 @@ void qc_stream_desc_free(struct qc_stream_desc *stream, int closing)
 	 */
 	frm_node = eb64_first(&stream->acked_frms);
 	while (frm_node) {
-		struct quic_stream *strm;
+		struct qf_stream *strm_frm;
 		struct quic_frame *frm;
 
-		strm = eb64_entry(frm_node, struct quic_stream, offset);
+		strm_frm = eb64_entry(frm_node, struct qf_stream, offset);
 
 		frm_node = eb64_next(frm_node);
-		eb64_delete(&strm->offset);
+		eb64_delete(&strm_frm->offset);
 
-		frm = container_of(strm, struct quic_frame, stream);
+		frm = container_of(strm_frm, struct quic_frame, stream);
 		qc_release_frm(qc, frm);
 	}
 
@@ -215,11 +215,12 @@ struct buffer *qc_stream_buf_get(struct qc_stream_desc *stream)
 	return &stream->buf->buf;
 }
 
-/* Returns the count of available buffer left for <qc>. */
+/* Check if a new stream buffer can be allocated for the connection <qc>.
+ * Returns a boolean.
+ */
 static int qc_stream_buf_avail(struct quic_conn *qc)
 {
-	BUG_ON(qc->stream_buf_count > global.tune.quic_streams_buf);
-	return global.tune.quic_streams_buf - qc->stream_buf_count;
+	return qc->stream_buf_count < global.tune.quic_streams_buf;
 }
 
 /* Allocate a new current buffer for <stream>. The buffer limit count for the
@@ -227,28 +228,26 @@ static int qc_stream_buf_avail(struct quic_conn *qc)
  * is not NULL prior to this call. The new buffer represents stream payload at
  * offset <offset>.
  *
- * Returns the buffer or NULL on error. Caller may check <avail> to ensure if
- * the connection buffer limit was reached or a fatal error was encountered.
+ * Returns the buffer or NULL.
  */
 struct buffer *qc_stream_buf_alloc(struct qc_stream_desc *stream,
-                                   uint64_t offset, int *avail)
+                                   uint64_t offset)
 {
 	struct quic_conn *qc = stream->qc;
 
 	/* current buffer must be released first before allocate a new one. */
 	BUG_ON(stream->buf);
 
-	*avail = qc_stream_buf_avail(qc);
-	if (!*avail)
+	if (!qc_stream_buf_avail(qc))
 		return NULL;
 
+	++qc->stream_buf_count;
+
 	stream->buf_offset = offset;
 	stream->buf = pool_alloc(pool_head_quic_stream_buf);
 	if (!stream->buf)
 		return NULL;
 
-	++qc->stream_buf_count;
-
 	stream->buf->buf = BUF_NULL;
 	LIST_APPEND(&stream->buf_list, &stream->buf->list);
 
